# tests/test_admin.py

import sys
import os
import pytest
import mysql.connector

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
sys.path.append(root_dir + '/src/classes')

from classes.Admin import Admin

DB_CONFIG = {
    'host': 'localhost',
    'database': 'warehosea',
    'user': 'root',
    'password': '170611'
}

@pytest.fixture(scope="module")
def db_connection():
    conn = mysql.connector.connect(**DB_CONFIG)
    yield conn
    conn.close()

@pytest.fixture
def admin():
    return Admin(surel="hoseaganteng@gmail.com", nama="Test User", password="password", notelp="1234567890")

@pytest.fixture
def setup_teardown_update(db_connection):
    cursor = db_connection.cursor()
    cursor.execute("SELECT surel, nama, password, noTelp FROM Admin WHERE surel = 'hoseaganteng@gmail.com'")
    original_data = cursor.fetchone()

    # If the admin does not exist, insert it first
    if original_data is None:
        cursor.execute("INSERT INTO Admin (surel, nama, password, noTelp) VALUES (%s, %s, %s, %s)", 
                       ('hoseaganteng@gmail.com', 'Original Name', 'originalpassword', '1234567890'))
        db_connection.commit()
        cursor.execute("SELECT surel, nama, password, noTelp FROM Admin WHERE surel = 'hoseaganteng@gmail.com'")
        original_data = cursor.fetchone()

    yield

    # Revert changes after test
    cursor.execute("UPDATE Admin SET nama = %s, password = %s, noTelp = %s WHERE surel = %s",
                   (original_data[1], original_data[2], original_data[3], original_data[0]))
    db_connection.commit()
    cursor.close()

def test_get_surel(admin):
    assert admin.get_surel() == "hoseaganteng@gmail.com"

def test_get_nama(admin):
    assert admin.get_nama() == "Test User"

def test_get_password(admin):
    assert admin.get_password() == "password"

def test_get_no_telp(admin):
    assert admin.get_no_telp() == "1234567890"

def test_set_nama(admin):
    admin.set_nama("New Name")
    assert admin.get_nama() == "New Name"

def test_set_password(admin):
    admin.set_password("newpassword")
    assert admin.get_password() == "newpassword"

def test_set_no_telp(admin):
    admin.set_no_telp("0987654321")
    assert admin.get_no_telp() == "0987654321"

def test_load_all_admins(db_connection):
    cursor = db_connection.cursor()
    cursor.execute("""
        INSERT INTO Admin (surel, nama, password, noTelp)
        VALUES (%s, %s, %s, %s)
        ON DUPLICATE KEY UPDATE
        nama = VALUES(nama), password = VALUES(password), noTelp = VALUES(noTelp)
    """, ("hoseacakep@gmail.com", "Hosea Ganteng", "securepassword", "1234567890"))
    db_connection.commit()

    admins = Admin.load_all_admins()
    assert len(admins) >= 1

    admin = next((admin for admin in admins if admin.get_surel() == "hoseacakep@gmail.com"), None)
    assert admin is not None
    assert admin.get_surel() == "hoseacakep@gmail.com"
    assert admin.get_nama() == "Hosea Ganteng"
    assert admin.get_password() == "securepassword"
    assert admin.get_no_telp() == "1234567890"

    cursor.execute("DELETE FROM Admin WHERE surel = %s", ("hoseacakep@gmail.com",))
    db_connection.commit()
    cursor.close()

def test_update_in_db(setup_teardown_update):
    admin = Admin(surel="hoseaganteng@gmail.com", nama="Admin One Updated", password="gtg", notelp="1111111112")
    admin.update_in_db()

    conn = mysql.connector.connect(**DB_CONFIG)
    cursor = conn.cursor()
    cursor.execute("SELECT surel, nama, password, noTelp FROM Admin WHERE surel = %s", ("hoseaganteng@gmail.com",))
    result = cursor.fetchone()
    conn.close()

    assert result[0] == "hoseaganteng@gmail.com"
    assert result[1] == "Admin One Updated"
    assert result[2] == "gtg"
    assert result[3] == "1111111112"

# Run the test
if __name__ == "__main__":
    pytest.main()
