# tests/test_warehouse.py

import sys
import os
import pytest
import mysql.connector

root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
sys.path.append(root_dir + '/src')

from classes.Warehouse import Warehouse

DB_CONFIG = {
    'host': 'localhost',
    'database': 'warehosea',
    'user': 'root',
    'password': '170611'
}

@pytest.fixture(scope="module")
def db_connection():
    conn = mysql.connector.connect(**DB_CONFIG)
    yield conn
    conn.close()

@pytest.fixture
def setup_teardown(db_connection):
    cursor = db_connection.cursor(dictionary=True)
    
    cursor.execute("SELECT * FROM warehouse WHERE id_warehouse = 1")
    original_data = cursor.fetchone()

    yield

    cursor.execute(
        "UPDATE warehouse SET nama_warehouse = %s, kapasitas_maksimum = %s, alamat = %s, kapasitas_terkini = %s WHERE id_warehouse = %s",
        (original_data['nama_warehouse'], original_data['kapasitas_maksimum'], original_data['alamat'], original_data['kapasitas_terkini'], original_data['id_warehouse'])
    )
    db_connection.commit()
    cursor.close()

def test_getters(db_connection):
    warehouse = Warehouse(db_connection, id_warehouse=1, nama_warehouse="Test Warehouse", kapasitas_maksimum=100, alamat="Test Address", kapasitas_terkini=50)
    
    assert warehouse.get_id_warehouse() == 1
    assert warehouse.get_nama_warehouse() == "Test Warehouse"
    assert warehouse.get_kapasitas_maksimum() == 100
    assert warehouse.get_alamat() == "Test Address"
    assert warehouse.get_kapasitas_terkini() == 50

def test_setters(db_connection):
    warehouse = Warehouse(db_connection)
    
    warehouse.set_id_warehouse(2)
    assert warehouse.get_id_warehouse() == 2
    
    warehouse.set_nama_warehouse("Updated Warehouse")
    assert warehouse.get_nama_warehouse() == "Updated Warehouse"
    
    warehouse.set_kapasitas_maksimum(200)
    assert warehouse.get_kapasitas_maksimum() == 200
    
    warehouse.set_alamat("Updated Address")
    assert warehouse.get_alamat() == "Updated Address"
    
    warehouse.set_kapasitas_terkini(100)
    assert warehouse.get_kapasitas_terkini() == 100

def test_update_warehouse(db_connection, setup_teardown):
    updates = {
        'nama_warehouse': 'Updated Warehouse',
        'kapasitas_maksimum': 200,
        'alamat': 'Updated Address',
        'kapasitas_terkini': 100
    }
    Warehouse.update(db_connection, 1, updates)
    
    cursor = db_connection.cursor()
    cursor.execute("SELECT nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini FROM warehouse WHERE id_warehouse = 1")
    result = cursor.fetchone()
    
    assert result[0] == 'Updated Warehouse'
    assert result[1] == 200
    assert result[2] == 'Updated Address'
    assert result[3] == 100
    cursor.close()

def test_search_warehouse(db_connection):
    # success case
    cursor = db_connection.cursor()
    cursor.execute("INSERT INTO warehouse (nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini, id_admin) VALUES (%s, %s, %s, %s, %s)", 
                   ("Test Search Warehouse", 100, "Test Address", 50, 'hoseaganteng@gmail.com'))
    db_connection.commit()
    results = Warehouse.search(db_connection, "Search Warehouse")
    
    assert len(results) > 0
    for result in results:
        assert "Search Warehouse" in result['nama_warehouse']
    
    # fail case
    results = Warehouse.search(db_connection, "Nonexistent Warehouse")
    assert len(results) == 0
    
    cursor.execute("DELETE FROM warehouse WHERE nama_warehouse = %s", ("Test Search Warehouse",))
    db_connection.commit()
    cursor.close()


def test_find_by_id(db_connection):
    result = Warehouse.find_by_id(db_connection, 1)
    
    assert result is not None
    assert result['id_warehouse'] == 1

def test_check_name(db_connection):
    cursor = db_connection.cursor()
    cursor.execute("INSERT INTO warehouse (nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini, id_admin) VALUES (%s, %s, %s, %s, %s)", 
                   ("Test Warehouse", 100, "Test Address", 50, 'hoseaganteng@gmail.com'))
    db_connection.commit()
    
    assert Warehouse.check_name(db_connection, "Test Warehouse") == True
    assert Warehouse.check_name(db_connection, "Nonexistent Warehouse") == False

    cursor.execute("DELETE FROM warehouse WHERE nama_warehouse = %s", ("Test Warehouse",))
    db_connection.commit()
    cursor.close()

def test_check_id(db_connection):
    assert Warehouse.check_id(db_connection, 1) == True
    assert Warehouse.check_id(db_connection, 9999) == False

def test_get_nama_from_id(db_connection):
    cursor = db_connection.cursor()
    cursor.execute("INSERT INTO warehouse (nama_warehouse, kapasitas_maksimum, alamat, kapasitas_terkini, id_admin) VALUES (%s, %s, %s, %s, %s)", 
                   ("Test Warehouse", 100, "Test Address", 50, 'hoseaganteng@gmail.com'))
    db_connection.commit()
    lastid = cursor.lastrowid
    
    result = Warehouse.get_nama_from_id(db_connection, lastid)
    
    assert result == "Test Warehouse"
    
    # Clean up
    cursor.execute("DELETE FROM warehouse WHERE id_warehouse = %s", (lastid,))
    db_connection.commit()
    cursor.close()

def test_get_num_warehouse(db_connection):
    result = Warehouse.get_num_warehouse(db_connection)
    
    assert result > 0

if __name__ == "__main__":
    pytest.main()
